@@ -16,10 +16,12 @@ extern "C" {
 #include <r_util/r_signal.h>
 #include <r_util/r_stack.h>
 #include <r_util/r_str.h>
+#include <r_util/r_str_constpool.h>
 #include <r_util/r_sys.h>
 #include <r_util/r_file.h>
 #include <r_vector.h>
 #include <sdb.h>
+#include <ht_up.h>
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -38,6 +40,9 @@ extern "C" {
 # ifndef ENABLE_VIRTUAL_TERMINAL_PROCESSING
 # define ENABLE_VIRTUAL_TERMINAL_PROCESSING 0x0004
 # endif
+# ifndef ENABLE_VIRTUAL_TERMINAL_INPUT
+# define ENABLE_VIRTUAL_TERMINAL_INPUT 0x0200
+# endif
 #else
 #include <unistd.h>
 #endif
@@ -57,12 +62,21 @@ extern "C" {
 
 R_LIB_VERSION_HEADER(r_cons);
 
+#define R_CONS_CMD_DEPTH 100
+
 typedef int (*RConsGetSize)(int *rows);
 typedef int (*RConsGetCursor)(int *rows);
+typedef bool (*RConsIsBreaked)(void);
+typedef void (*RConsFlush)(void);
+typedef void (*RConsGrepCallback)(const char *grep);
 
 typedef struct r_cons_bind_t {
 	RConsGetSize get_size;
 	RConsGetCursor get_cursor;
+	PrintfCallback cb_printf;
+	RConsIsBreaked is_breaked;
+	RConsFlush cb_flush;
+	RConsGrepCallback cb_grep;
 } RConsBind;
 
 typedef struct r_cons_grep_t {
@@ -72,6 +86,8 @@ typedef struct r_cons_grep_t {
 	int counter;
 	bool charCounter;
 	int less;
+	bool hud;
+	bool human;
 	int json;
 	char *json_path;
 	int range_line;
@@ -84,80 +100,21 @@ typedef struct r_cons_grep_t {
 	int tokens[R_CONS_GREP_TOKENS];
 	int tokens_used;
 	int amp;
+	int zoom;
+	int zoomy; // if set then its scaled unproportionally
 	int neg;
 	int begin;
 	int end;
 	int icase;
 } RConsGrep;
 
-#if 0
-// TODO Might be better than using r_cons_pal_get_i
-// And have smaller RConsPrintablePalette and RConsPalette
-enum {
-	R_CONS_PAL_0x00 = 0,
-	R_CONS_PAL_0x7f,
-	R_CONS_PAL_0xff,
-	R_CONS_PAL_ARGS,
-	R_CONS_PAL_BIN,
-	R_CONS_PAL_BTEXT,
-	R_CONS_PAL_CALL,
-	R_CONS_PAL_CJMP,
-	R_CONS_PAL_CMP,
-	R_CONS_PAL_COMMENT,
-	R_CONS_PAL_CREG,
-	R_CONS_PAL_FLAG,
-	R_CONS_PAL_FLINE,
-	R_CONS_PAL_FLOC,
-	R_CONS_PAL_FLOW,
-	R_CONS_PAL_FLOW2,
-	R_CONS_PAL_FNAME,
-	R_CONS_PAL_HELP,
-	R_CONS_PAL_INPUT,
-	R_CONS_PAL_INVALID,
-	R_CONS_PAL_JMP,
-	R_CONS_PAL_LABEL,
-	R_CONS_PAL_MATH,
-	R_CONS_PAL_MOV,
-	R_CONS_PAL_NOP,
-	R_CONS_PAL_NUM,
-	R_CONS_PAL_OFFSET,
-	R_CONS_PAL_OTHER,
-	R_CONS_PAL_POP,
-	R_CONS_PAL_PROMPT,
-	R_CONS_PAL_PUSH,
-	R_CONS_PAL_CRYPTO,
-	R_CONS_PAL_REG,
-	R_CONS_PAL_RESET,
-	R_CONS_PAL_RET,
-	R_CONS_PAL_SWI,
-	R_CONS_PAL_TRAP,
-	R_CONS_PAL_AI_READ,
-	R_CONS_PAL_AI_WRITE,
-	R_CONS_PAL_AI_EXEC,
-	R_CONS_PAL_AI_SEQ,
-	R_CONS_PAL_AI_ASCII,
-	R_CONS_PAL_AI_UNMAP,
-	R_CONS_PAL_GUI_CFLOW,
-	R_CONS_PAL_GUI_DATAOFFSET,
-	R_CONS_PAL_GUI_BACKGROUND,
-	R_CONS_PAL_GUI_ALT_BACKGROUND,
-	R_CONS_PAL_GUI_BORDER,
-	R_CONS_PAL_LINEHL,
-	R_CONS_PAL_GRAPH_BOX,
-	R_CONS_PAL_GRAPH_BOX2,
-	R_CONS_PAL_GRAPH_BOX3,
-	R_CONS_PAL_GRAPH_BOX4,
-	R_CONS_PAL_GRAPH_TRUE,
-	R_CONS_PAL_GRAPH_FALSE,
-	R_CONS_PAL_GRAPH_TRUFAE,
-	R_CONS_PAL_GRAPH_TRACED,
-	R_CONS_PAL_GRAPH_CURRENT,
-	R_CONS_PAL_LAST
-};
-#endif
-
 enum { ALPHA_RESET = 0x00, ALPHA_FG = 0x01, ALPHA_BG = 0x02, ALPHA_FGBG = 0x03 };
-enum { R_CONS_ATTR_BOLD = 1 << 1 };
+enum { R_CONS_ATTR_BOLD = 1u << 1,
+       R_CONS_ATTR_DIM = 1u << 2,
+       R_CONS_ATTR_ITALIC = 1u << 3,
+       R_CONS_ATTR_UNDERLINE = 1u << 4,
+       R_CONS_ATTR_BLINK = 1u << 5
+};
 
 typedef struct rcolor_t {
 	// bold, italic, underline, ...
@@ -241,6 +198,10 @@ typedef struct r_cons_palette_t {
 	RColor graph_trufae;
 	RColor graph_traced;
 	RColor graph_current;
+        RColor graph_diff_match;
+        RColor graph_diff_unmatch;
+        RColor graph_diff_unknown;
+        RColor graph_diff_new;
 } RConsPalette;
 
 typedef struct r_cons_printable_palette_t {
@@ -308,6 +269,10 @@ typedef struct r_cons_printable_palette_t {
 	char *graph_box2;
 	char *graph_box3;
 	char *graph_box4;
+	char *graph_diff_match;
+	char *graph_diff_unmatch;
+	char *graph_diff_unknown;
+	char *graph_diff_new;
 	char *graph_true;
 	char *graph_false;
 	char *graph_trufae;
@@ -320,11 +285,6 @@ typedef struct r_cons_printable_palette_t {
 typedef void (*RConsEvent)(void *);
 
 #define CONS_MAX_ATTR_SZ 16
-typedef struct r_cons_canvas_attr_t {
-	//TODO add support for 256 colors.
-	int loc;
-	const char * a;
-} RConsCanvasAttr;
 
 typedef struct r_cons_canvas_t {
 	int w;
@@ -334,9 +294,9 @@ typedef struct r_cons_canvas_t {
 	char **b;
 	int *blen;
 	int *bsize;
-	const char * attr;//The current attr (inserted on each write)
-	RConsCanvasAttr * attrs;// all the different attributes
-	int attrslen;
+	const char *attr; //The current attr (inserted on each write)
+	HtUP *attrs; // all the different attributes <key: unsigned int loc, const char *attr>
+	RStrConstPool constpool; // Pool for non-compile-time attrs
 	int sx; // scrollx
 	int sy; // scrolly
 	int color;
@@ -391,6 +351,21 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CURVE_CORNER_TR "╮"
 #define RUNE_CURVE_CORNER_BR "╯"
 #define RUNE_CURVE_CORNER_BL "╰"
+#define RUNE_LONG_LINE_HORIZ "―"
+#define UTF_CIRCLE "\u25EF"
+#define UTF_BLOCK "\u2588"
+
+// Emoji
+#define UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
+#define UTF8_WHITE_HEAVY_CHECK_MARK "✅"
+#define UTF8_SEE_NO_EVIL_MONKEY "🙈"
+#define UTF8_SKULL_AND_CROSSBONES "☠"
+#define UTF8_KEYBOARD "⌨"
+#define UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
+#define UTF8_DOOR "🚪"
+
+// Variation Selectors
+#define UTF8_VS16 "\xef\xb8\x8f"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -398,6 +373,7 @@ typedef void (*RConsBreakCallback)(void *core);
 typedef void *(*RConsSleepBeginCallback)(void *core);
 typedef void (*RConsSleepEndCallback)(void *core, void *user);
 typedef void (*RConsQueueTaskOneshot)(void *core, void *task, void *user);
+typedef void (*RConsFunctionKey)(void *core, int fkey);
 
 typedef enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MODE_16M } RConsColorMode;
 
@@ -405,13 +381,14 @@ typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
 	char *buffer;
-	int buffer_len;
-	int buffer_sz;
+	size_t buffer_len;
+	size_t buffer_sz;
 
 	bool breaked;
 	RStack *break_stack;
 	RConsEvent event_interrupt;
 	void *event_interrupt_data;
+	int cmd_depth;
 
 	// Used for per-task logging redirection
 	RLogCallback log_callback; // TODO: RList of callbacks
@@ -428,10 +405,18 @@ typedef struct r_cons_context_t {
 	RConsPrintablePalette pal;
 } RConsContext;
 
+#define HUD_BUF_SIZE 512
+
+typedef struct {
+	int x;
+	int y;
+} RConsCursorPos;
+
 typedef struct r_cons_t {
 	RConsContext *context;
 	char *lastline;
-	int is_html;
+	bool is_html;
+	bool was_html;
 	int lines;
 	int rows;
 	int echo; // dump to stdout in realtime
@@ -458,12 +443,14 @@ typedef struct r_cons_t {
 	RConsSleepEndCallback cb_sleep_end;
 	RConsClickCallback cb_click;
 	RConsQueueTaskOneshot cb_task_oneshot;
+	RConsFunctionKey cb_fkey;
 
 	void *user; // Used by <RCore*>
 #if __UNIX__
 	struct termios term_raw, term_buf;
 #elif __WINDOWS__
-	DWORD term_raw, term_buf;
+	DWORD term_raw, term_buf, term_xterm;
+	UINT old_cp;
 #endif
 	RNum *num;
 	/* Pager (like more or less) to use if the output doesn't fit on the
@@ -471,16 +458,15 @@ typedef struct r_cons_t {
 	char *pager;
 	int blankline;
 	char *highlight;
+	bool enable_highlight;
 	int null; // if set, does not show anything
 	int mouse;
 	int is_wine;
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
-	bool newline;
-#if __WINDOWS__
-	int ansicon;
-#endif
+	R_DEPRECATE bool newline;
+	int vtmode;
 	bool flush;
 	bool use_utf8; // use utf8 features
 	bool use_utf8_curvy; // use utf8 curved corners
@@ -489,26 +475,20 @@ typedef struct r_cons_t {
 	int pagesize;
 	char *break_word;
 	int break_word_len;
-	ut64 timeout;
+	ut64 timeout; // must come from r_time_now_mono()
 	bool grep_color;
 	bool grep_highlight;
 	bool use_tts;
 	bool filter;
 	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
+	bool click_set;
+	int click_x;
+	int click_y;
+	bool show_vals;		// show which section in Vv
 	// TODO: move into instance? + avoid unnecessary copies
+	RConsCursorPos cpos;
 } RCons;
 
-// XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
-/* XXX : global variables? or a struct with a singleton? */
-//extern FILE *stdin_fd;
-//extern FILE *r_cons_stdin_fd;
-//extern int r_cons_stdout_fd;
-//extern int r_cons_stdout_file;
-//extern char *r_cons_filterline;
-//extern char *r_cons_teefile;
-// not needed anymoar
-//extern int (*r_cons_user_fgets)(char *buf, int len);
-
 #define R_CONS_KEY_F1 0xf1
 #define R_CONS_KEY_F2 0xf2
 #define R_CONS_KEY_F3 0xf3
@@ -526,12 +506,15 @@ typedef struct r_cons_t {
 
 #define R_CONS_CLEAR_LINE "\x1b[2K\r"
 #define R_CONS_CLEAR_SCREEN "\x1b[2J\r"
-#define R_CONS_ADD_NEWLINES "\x1b[2J"
 #define R_CONS_CLEAR_FROM_CURSOR_TO_END "\x1b[0J\r"
 
 #define R_CONS_CURSOR_SAVE "\x1b[s"
 #define R_CONS_CURSOR_RESTORE "\x1b[u"
 #define R_CONS_GET_CURSOR_POSITION "\x1b[6n"
+#define R_CONS_CURSOR_UP "\x1b[A"
+#define R_CONS_CURSOR_DOWN "\x1b[B"
+#define R_CONS_CURSOR_RIGHT "\x1b[C"
+#define R_CONS_CURSOR_LEFT "\x1b[D"
 
 #define Color_BLINK        "\x1b[5m"
 #define Color_INVERT       "\x1b[7m"
@@ -701,6 +684,11 @@ enum {
 	LINE_NOSYM_HORIZ
 };
 
+typedef enum {
+	INSERT_MODE = 'i',
+	CONTROL_MODE = 'c'
+} RViMode;
+
 #define DOT_STYLE_NORMAL 0
 #define DOT_STYLE_CONDITIONAL 1
 #define DOT_STYLE_BACKEDGE 2
@@ -739,6 +727,7 @@ typedef struct r_cons_canvas_line_style_t {
 
 
 #ifdef R_API
+R_API void r_cons_image(const ut8 *buf, int bufsz, int width, int mode);
 R_API RConsCanvas* r_cons_canvas_new(int w, int h);
 R_API void r_cons_canvas_free(RConsCanvas *c);
 R_API void r_cons_canvas_clear(RConsCanvas *c);
@@ -750,6 +739,7 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s);
 R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y);
 R_API void r_cons_canvas_goto_write(RConsCanvas *c,int x,int y, const char * s);
 R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color);
+R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, const char *color);
 R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
@@ -762,26 +752,36 @@ R_API RCons *r_cons_singleton(void);
 R_API RCons *r_cons_free(void);
 R_API char *r_cons_lastline(int *size);
 R_API char *r_cons_lastline_utf8_ansi_len(int *len);
+R_API void r_cons_set_click(int x, int y);
+R_API bool r_cons_get_click(int *x, int *y);
 
 typedef void (*RConsBreak)(void *);
-R_API void r_cons_break_end(void);
 R_API bool r_cons_is_breaked(void);
 R_API bool r_cons_is_interactive(void);
 R_API bool r_cons_default_context_is_interactive(void);
-R_API void r_cons_break_timeout(int timeout);
-R_API void r_cons_breakword(const char *s);
 R_API void *r_cons_sleep_begin(void);
 R_API void r_cons_sleep_end(void *user);
 
+/* ^C */
+R_API void r_cons_break_push(RConsBreak cb, void *user);
+R_API void r_cons_break_pop(void);
+R_API void r_cons_break_clear(void);
+R_API void r_cons_breakword(const char *s);
+R_API void r_cons_break_end(void);
+R_API void r_cons_break_timeout(int timeout);
+
 /* pipe */
 R_API int r_cons_pipe_open(const char *file, int fdn, int append);
 R_API void r_cons_pipe_close(int fd);
 
 #if __WINDOWS__
-R_API bool r_cons_is_ansicon(void);
+R_API int r_cons_is_vtcompat(void);
+R_API void r_cons_w32_clear(void);
 R_API void r_cons_w32_gotoxy(int fd, int x, int y);
-R_API int r_cons_w32_print(const ut8 *ptr, int len, bool vmode);
-R_API int r_cons_win_printf(bool vmode, const char *fmt, ...);
+R_API int r_cons_w32_print(const char *ptr, int len, bool vmode);
+R_API int r_cons_win_printf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
+R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
+R_API int r_cons_win_vhprintf(DWORD hdl, bool vmode, const char *fmt, va_list ap);
 #endif
 
 R_API void r_cons_push(void);
@@ -794,15 +794,13 @@ R_API bool r_cons_context_is_main(void);
 R_API void r_cons_context_break(RConsContext *context);
 R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void *user, bool sig);
 R_API void r_cons_context_break_pop(RConsContext *context, bool sig);
-R_API void r_cons_break_push(RConsBreak cb, void *user);
-R_API void r_cons_break_pop(void);
-R_API void r_cons_break_clear(void);
 
 /* control */
 R_API char *r_cons_editor(const char *file, const char *str);
 R_API void r_cons_reset(void);
 R_API void r_cons_reset_colors(void);
 R_API void r_cons_print_clear(void);
+R_API void r_cons_echo(const char *msg);
 R_API void r_cons_zero(void);
 R_API void r_cons_highlight(const char *word);
 R_API void r_cons_clear(void);
@@ -821,9 +819,11 @@ R_API void r_cons_chop(void);
 R_API void r_cons_set_raw(bool b);
 R_API void r_cons_set_interactive(bool b);
 R_API void r_cons_set_last_interactive(void);
+R_API void r_cons_set_utf8(bool b);
+R_API void r_cons_grep(const char *grep);
 
 /* output */
-R_API int r_cons_printf(const char *format, ...);
+R_API int r_cons_printf(const char *format, ...) R_PRINTF_CHECK(1, 2);
 R_API void r_cons_printf_list(const char *format, va_list ap);
 R_API void r_cons_strcat(const char *str);
 R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
@@ -831,10 +831,12 @@ R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
 R_API void r_cons_println(const char* str);
 
 R_API void r_cons_strcat_justify(const char *str, int j, char c);
+R_API void r_cons_printat(const char *str, int x, char y);
 R_API int r_cons_memcat(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
+R_API void r_cons_print_fps (int col);
 R_API void r_cons_last(void);
 R_API int r_cons_less_str(const char *str, const char *exitkeys);
 R_API void r_cons_less(void);
@@ -842,10 +844,10 @@ R_API void r_cons_2048(bool color);
 R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write(char *buffer);
-R_API int r_cons_is_utf8(void);
+R_API bool r_cons_is_utf8(void);
 R_API void r_cons_cmd_help(const char * help[], bool use_color);
 R_API void r_cons_log_stub(const char *output, const char *funcname, const char *filename,
- unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...);
+ unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...) R_PRINTF_CHECK(7, 8);
 
 
 /* input */
@@ -908,15 +910,16 @@ R_API void r_cons_color(int fg, int r, int g, int b);
 
 R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
-R_API bool r_cons_yesno(int def, const char *fmt, ...);
+R_API bool r_cons_yesno(int def, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_cons_input(const char *msg);
 R_API char *r_cons_password(const char *msg);
-R_API void r_cons_set_cup(int enable);
+R_API bool r_cons_set_cup(bool enable);
 R_API void r_cons_column(int c);
 R_API int r_cons_get_column(void);
 R_API char *r_cons_message(const char *msg);
 R_API void r_cons_set_title(const char *str);
 R_API bool r_cons_enable_mouse(const bool enable);
+R_API void r_cons_enable_highlight(const bool enable);
 R_API void r_cons_bind(RConsBind *bind);
 R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
@@ -956,6 +959,13 @@ typedef struct r_line_buffer_t {
 	int length;
 } RLineBuffer;
 
+typedef struct r_hud_t {
+	int current_entry_n;
+	int top_entry_n;
+	char activate;
+	int vi;
+} RLineHud;
+
 typedef struct r_line_t RLine; // forward declaration
 typedef struct r_line_comp_t RLineCompletion;
 
@@ -966,6 +976,7 @@ typedef int (*RLineCompletionCb)(RLineCompletion *completion, RLineBuffer *buf,
 struct r_line_comp_t {
 	bool opt;
 	size_t args_limit;
+	bool quit;
 	RPVector args; /* <char *> */
 	RLineCompletionCb run;
 	void *run_user;
@@ -984,10 +995,14 @@ struct r_line_t {
 	RLineHistoryUpCb cb_history_up;
 	RLineHistoryDownCb cb_history_down;
 	RLineEditorCb cb_editor;
+	// RLineFunctionKeyCb cb_fkey;
+	RConsFunctionKey cb_fkey;
 	/* state , TODO: use more bool */
 	int echo;
 	int has_echo;
 	char *prompt;
+	RList/*<str>*/ *kill_ring;
+	int kill_ring_ptr;
 	char *clipboard;
 	int disable;
 	void *user;
@@ -995,14 +1010,16 @@ struct r_line_t {
 	int (*hist_down)(void *user);
 	char *contents;
 	bool zerosep;
+	bool enable_vi_mode;
+	int vi_mode;
+	bool prompt_mode;
 	RLinePromptType prompt_type;
 	int offset_hist_index;
 	int file_hist_index;
+	RLineHud *hud;
 	RList *sdbshell_hist;
 	RListIter *sdbshell_hist_iter;
-#if __WINDOWS__
-	int ansicon;
-#endif
+	int vtmode;
 }; /* RLine */
 
 #ifdef R_API
@@ -1013,6 +1030,7 @@ R_API void r_line_free(void);
 R_API char *r_line_get_prompt(void);
 R_API void r_line_set_prompt(const char *prompt);
 R_API int r_line_dietline_init(void);
+R_API void r_line_clipboard_push (const char *str);
 R_API void r_line_hist_free(void);
 
 typedef int (RLineReadCallback)(void *user, const char *line);
@@ -1042,13 +1060,11 @@ R_API void r_line_completion_clear(RLineCompletion *completion);
 #endif
 
 typedef int (*RPanelsMenuCallback)(void *user);
-typedef char *(*RPanelsMenuGetName)(R_NULLABLE void *user, char *base_name);
 typedef struct r_panels_menu_item {
 	int n_sub, selectedIndex;
-	char *base_name;
+	char *name;
 	struct r_panels_menu_item **sub;
 	RPanelsMenuCallback cb;
-	RPanelsMenuGetName get_name_cb;
 	RPanel *p;
 } RPanelsMenuItem;
 
@@ -1097,12 +1113,15 @@ typedef struct r_panels_t {
 	int n_panels;
 	int columnWidth;
 	int curnode;
-	bool isResizing;
+	int mouse_orig_x;
+	int mouse_orig_y;
 	bool autoUpdate;
-	RPanelsMenu *panelsMenu;
+	bool mouse_on_edge_x;
+	bool mouse_on_edge_y;
+	RPanelsMenu *panels_menu;
 	Sdb *db;
 	Sdb *rotate_db;
-	Sdb *almighty_db;
+	Sdb *modal_db;
 	HtPP *mht;
 	RPanelsMode mode;
 	RPanelsFun fun;
@@ -1122,12 +1141,24 @@ typedef enum {
 typedef struct r_panels_root_t {
 	int n_panels;
 	int cur_panels;
+	Sdb *pdc_caches;
+	Sdb *cur_pdc_cache;
 	RPanels **panels;
 	RPanelsRootState root_state;
 } RPanelsRoot;
 
-#ifdef __cplusplus
+
+#ifdef __sun
+static inline void cfmakeraw(struct termios *tm) {
+	tm->c_cflag &= ~(CSIZE | PARENB);
+	tm->c_cflag |= CS8;
+	tm->c_iflag &= ~(IMAXBEL | IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
+	tm->c_oflag &= ~OPOST;
+	tm->c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
 }
 #endif
 
+#ifdef __cplusplus
+}
+#endif
 #endif
